#!/bin/bash
VER=1.1.2

config=/glftpd/bin/mss.conf

##############################################
# GO                                         #
##############################################

. $config

if [ "$1" = "debug" -o "$2" = "debug" ]; then
  DEBUG=TRUE
fi

if [ "$HIGHDEBUG" = "TRUE" ]; then
  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Starting MSS v$VER\" >> $LOG
fi

if [ ! -e $USERSOURCE/$VERIFYUSER ]; then
  if [ "$REMOUNT" = "TRUE" ]; then
    VERFSTAB="$( grep -w "$USERSOURCE" /etc/fstab )"
    if [ "$VERFSTAB" = "" ]; then
      echo "REMOUNT is TRUE, but no entry in /etc/fstab for $USERSOURCE. Quitting."
      exit 0 
    fi
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Attempting to remount $USERSSOURCE.\" >> $LOG
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Attempting to remount $USERSSOURCE."
    fi
    umount -l $USERSOURCE >/dev/null 2>&1
    umount $USERSOURCE >/dev/null 2>&1
    sleep 1
    mount $USERSOURCE >/dev/null 2>&1
    if [ -e $USERSOURCE/$VERIFYUSER ]; then
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Verify user $VERIFYUSER does not exist. Remount successfull.\" >> $LOG
      fi
    else
      echo "Verify user $VERIFYUSER does not exist. Remount failed."
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Verify user $VERIFYUSER does not exist. Remount failed.\" >> $LOG
      fi
      exit 0
    fi
  else
    echo "Verify user $VERIFYUSER on source does not exist - Link down?"
    exit 0
  fi
fi

if [ ! -e $PASSWDSOURCE/passwd ]; then
  if [ "$REMOUNT" = "TRUE" ]; then
    VERFSTAB="$( grep -w "$PASSWDSOURCE" /etc/fstab )"
    if [ "$VERFSTAB" = "" ]; then
      echo "REMOUNT is TRUE, but no entry in /etc/fstab for $USERSOURCE. Quitting."
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"REMOUNT is TRUE, but no entry in /etc/fstab for $USERSOURCE. Quitting.\" >> $LOG
      fi
      exit 0 
    fi
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Attempting to remount $PASSWDSOURCE.\" >> $LOG
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Attempting to remount $PASSWDSOURCE."
    fi
    umount -l $PASSWDSOURCE >/dev/null 2>&1
    umount $PASSWDSOURCE >/dev/null 2>&1
    sleep 1
    mount $PASSWDSOURCE >/dev/null 2>&1
    if [ -e $PASSWDSOURCE/passwd ]; then
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Cant find $PASSWDSOURCE/passwd - Remount successfull.\" >> $LOG
      fi
    else
      echo "Cant find $PASSWDSOURCE/passwd - Remount failed."
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Cant find $PASSWDSOURCE/passwd - Remount failed.\" >> $LOG
      fi
      exit 0
    fi
  else
    echo "Cant find $PASSWDSOURCE/passwd - Link down?"
    exit 0
  fi
fi

if [ ! -r $PASSWDSOURCE/passwd ]; then
  echo "Cant read $PASSWDSOURCE/passwd - Wrong perms? Quitting."
  exit 0
fi

if [ ! -r $PASSWDSOURCE/group ]; then
  echo "Cant read $PASSWDSOURCE/group - Wrong perms? Quitting."
  exit 0
fi

if [ "$LOG" != "" ]; then
  if [ -e $LOG ]; then
    if [ "$PERMS" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Setting $PERMS on $LOG\" >> $LOG
      fi
      chmod 777 $LOG >/dev/null 2>&1
    fi
  else
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Touching $LOG\" >> $LOG
    fi
    touch $LOG
  fi
fi

if [ -e "$LOCKFILE" ]; then
  if [ "$LOG" != "" ]; then
    echo `date "+%a %b %e %T %Y"` USYNC: \"Error: Lockfile exists. Not running.\" >> $LOG
  fi
  if [ "$DEBUG" = "TRUE" ]; then
    echo "Autosync error: $LOCKFILE exists"
  fi
  exit 0
fi

if [ "$LOCKFILE" != "" ]; then
  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Touching lockfile: $LOCKFILE\" >> $LOG
  fi
  touch $LOCKFILE
fi

if [ "$DEBUG" = "TRUE" ]; then
  echo "Network seems fine."
fi

## Copy files to local computer if its on.
BFILE=""
if [ "$RUNLOCAL" = "TRUE" ]; then
  if [ ! -e $TMP/autosync.source.users ]; then
    mkdir $TMP/autosync.source.users
    chmod $PERMS $TMP/autosync.source.users >/dev/null 2>&1
  fi

  if [ "$BACKUPDEST" != "" ]; then
    BFILE="$( date +%m%d%y"-"%H%M )"
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Backing up previous source user files to $BACKUPDEST/autosync.source.$BFILE"
    fi
    tar -cf $BACKUPDEST/autosync.source.$BFILE".tar" $TMP/autosync.source.users >/dev/null 2>&1
    if [ "$ZIP" = "TRUE" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Zipping previous source userfiles to $BACKUPDEST/autosync.source.$BFILE".tar.gz"\" >> $LOG
      fi
      zip $BACKUPDEST/autosync.source.$BFILE".tar.gz" $BACKUPDEST/autosync.source.$BFILE".tar" >/dev/null 2>&1
      rm -f $BACKUPDEST/autosync.source.$BFILE".tar"
    fi
  fi
  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Removing all old files in $TMP/autosync.source.users\" >> $LOG
  fi
  rm -f $TMP/autosync.source.users/*

  if [ "$DEBUG" = "TRUE" ]; then
    echo "Copying $USERSOURCE files to $TMP/autosync.source.users to run from there."
  fi
  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Copying $USERSOURCE files to $TMP/autosync.source.users\" >> $LOG
  fi
  if [ "$DEBUG" = "TRUE" ]; then
    cp -f -v $USERSOURCE/* $TMP/autosync.source.users
  else
    cp -f $USERSOURCE/* $TMP/autosync.source.users >/dev/null 2>&1
  fi
  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Copy complete. Setting USERSOURCE to $TMP/autosync.source.users\" >> $LOG
  fi
  USERSOURCEORG="$USERSOURCE"
  USERSOURCE=$TMP/autosync.source.users
fi  

if [ "$BACKUPDEST" != "" ]; then
  if [ -w "$BACKUPDEST" ]; then
    if [ "$BFILE" = "" ]; then
      BFILE="$( date +%m%d%y"-"%H%M )"
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Backing up destination files to $BACKUPDEST/autosync.$BFILE"
    fi
    tar -cf $BACKUPDEST/autosync.$BFILE".tar" $USERDEST/* $PASSWDDEST/group $PASSWDDEST/passwd >/dev/null 2>&1
    if [ "$ZIP" = "TRUE" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Zipping present userfiles to $BACKUPDEST/autosync.$BFILE".tar.gz"\" >> $LOG
      fi
      zip $BACKUPDEST/autosync.$BFILE".tar.gz" $BACKUPDEST/autosync.$BFILE".tar" >/dev/null 2>&1
      rm -f $BACKUPDEST/autosync.$BFILE".tar"
    fi
    amount="$( ls $BACKUPDEST | wc -l | tr -d ' ' )"
    if [ -e $BACKUPDEST/deleted ]; then
      MAXBACKUPS="$( expr $MAXBACKUPS \+ 1 )"
    fi
    if [ "$amount" -gt "$MAXBACKUPS" ]; then
      del="$( expr $amount \- $MAXBACKUPS )"
      if [ "$del" -gt "0" ]; then
        for old in `ls -tl $BACKUPDEST | grep -v deleted | tail -n $del | awk '{print $9}'`; do
          if [ "$old" != "" ]; then
            rm -f $BACKUPDEST/$old
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"More then $MAXBACKUPS backups. Deleting old backup: $old\" >> $LOG
            fi
            if [ "$DEBUG" = "TRUE" ]; then
              echo "Deleting old backup: $old"
            fi
          fi
        done
      fi
    fi
  fi
fi

if [ ! -r $USERSOURCE/$VERIFYUSER ]; then
  echo "Verify user $VERIFYUSER on source does not exist - Link down?"
  exit 0
fi

if [ "$PERMS" != "" ]; then
  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Setting $PERMS on $USERSOURCE\" >> $LOG
  fi
  if [ "$DEBUG" = "TRUE" ]; then
    echo "Settings permissions on $USERSOURCE."
  fi
  chmod $PERMS $USERSOURCE/* >/dev/null 2>&1

  if [ "$USERSOURCEORG" != "" ]; then
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Setting $PERMS on $USERSOURCEORG\" >> $LOG
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Settings permissions on $USERSOURCEORG."
    fi
    chmod $PERMS $USERSOURCEORG/* >/dev/null 2>&1
  fi

  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Setting $PERMS on $USERDEST\" >> $LOG
  fi
  if [ "$DEBUG" = "TRUE" ]; then
    echo "Settings permissions on $USERDEST."
  fi
  chmod $PERMS $USERDEST/* >/dev/null 2>&1

  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Setting $PERMS on $TMP\" >> $LOG
  fi
  if [ "$DEBUG" = "TRUE" ]; then
    echo "Settings permissions on $TMP"
  fi
  chmod -R $PERMS $TMP >/dev/null 2>&1
fi  

####### STATS ########################################################

## Move stats from destination to source.
if [ "$1" = "stats" ]; then
  SKIP="NO"

  if [ "$STATSLOG" != "" ]; then
    if [ ! -w "$STATLOG" ]; then
      touch $STATLOG
      chmod 777 $STATLOG >/dev/null 2>&1
    fi
  fi

  cd $USERDEST
  for USER in `grep -w FLAGS * | awk '{print $1}' | awk -F":" '{print $1}' | egrep -v $EXCLUDE`; do

    ## Make sure TAGLINE does not contain any of the words in the below egrep.
    VERIFY=""
    VERIFY="$( grep -w TAGLINE $USER | egrep -w 'ALLUP|ALLDN|WKUP|WKDN|DAYUP|DAYDN|MONTHUP|MONTHDN|NUKE|TIME' )"
    if [ "$VERIFY" != "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Warning: TAGLINE of $USER contains a bad word. This can cause probs. Not moving stats."
      fi
      if [ "$STATLOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Warning: TAGLINE of $USER contains a bad word. Not moving stats\" >> $STATLOG
      fi
      SKIP="YES"
    fi

    if [ "$DEBUG" = "TRUE" ]; then
      echo "Moving Stats from $USER"
    fi

    ## ALLUP
    if [ "$MOVEALLUP" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTALLUP="$( grep -w ALLUP $USERDEST/$USER )"
        if [ "$DESTALLUP" != "" ]; then
          DESTALLUP1="$( echo $DESTALLUP | awk '{print $2}' )"
          DESTALLUP2="$( echo $DESTALLUP | awk '{print $3}' )"
          DESTALLUP3="$( echo $DESTALLUP | awk '{print $4}' )"
          if [ "$DESTALLUP1" = "" ]; then 
          DESTALLUP1="0"
          fi
          if [ "$DESTALLUP3 = "" ]; then
            DESTALLUP2="0"
          fi
          if [ "$DESTALLUP3 = "" ]; then
            DESTALLUP3="0"
          fi
          if [ "$DESTALLUP1" != "0" -a "$DESTALLUP2" != "0" -a "$DESTALLUP3" != "0" ]; then
            SOURCEALLUP="$( grep -w ALLUP $USERSOURCE/$USER )"
            if [ "$SOURCEALLUP" != "" ]; then
              SOURCEALLUP1="$( echo $SOURCEALLUP | awk '{print $2}' )"
              SOURCEALLUP2="$( echo $SOURCEALLUP | awk '{print $3}' )"
              SOURCEALLUP3="$( echo $SOURCEALLUP | awk '{print $4}' )"
              if [ "$SOURCEALLUP1" = "" ]; then
                SOURCEALLUP1="0"
              fi
              if [ "$SOURCEALLUP3 = "" ]; then
                SOURCEALLUP2="0"
              fi
              if [ "$SOURCEALLUP3 = "" ]; then
                SOURCEALLUP3="0"
              fi
              NEWALLUP1="$( expr "$SOURCEALLUP1" \+ "$DESTALLUP1" )"
              NEWALLUP2="$( expr "$SOURCEALLUP2" \+ "$DESTALLUP2" )"
              NEWALLUP3="$( expr "$SOURCEALLUP3" \+ "$DESTALLUP3" )"
              if [ "$NEWALLUP1" != "" -a "$NEWALLUP2" != "" -a "$NEWALLUP3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER ALLUP Stats $DESTNAME : $DESTALLUP1 $DESTALLUP2  $DESTALLUP3"
                  echo "$USER ALLUP Stats $SOURCENAME : $SOURCEALLUP1 $SOURCEALLUP2 $SOURCEALLUP3"
                  echo "$USER ALLUP Stats NEW : $NEWALLUP1 $NEWALLUP2 $NEWALLUP3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME ALLUP : $DESTALLUP1 $DESTALLUP2 $DESTALLUP3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME ALLUP : $SOURCEALLUP1 $SOURCEALLUP2 $SOURCEALLUP3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new ALLUP stats: $NEWALLUP1 $NEWALLUP2 $NEWALLUP3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^ALLUP.*/ALLUP $NEWALLUP1 $NEWALLUP2 $NEWALLUP3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^ALLUP.*/ALLUP $NEWALLUP1 $NEWALLUP2 $NEWALLUP3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^ALLUP.*/ALLUP 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^ALLUP.*/ALLUP 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new ALLUP stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read ALLUP from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching ALLUP on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read ALLUP from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## ALLDN
    if [ "$MOVEALLDN" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTALLDN="$( grep -w ALLDN $USERDEST/$USER )"
        if [ "$DESTALLDN" != "" ]; then
          DESTALLDN1="$( echo $DESTALLDN | awk '{print $2}' )"
          DESTALLDN2="$( echo $DESTALLDN | awk '{print $3}' )"
          DESTALLDN3="$( echo $DESTALLDN | awk '{print $4}' )"
          if [ "$DESTALLDN1" = "" ]; then
            DESTALLDN1="0"
          fi
          if [ "$DESTALLDN3 = "" ]; then
            DESTALLDN2="0"
          fi
          if [ "$DESTALLDN3 = "" ]; then
            DESTALLDN3="0"
          fi
          if [ "$DESTALLDN1" != "0" -a "$DESTALLDN2" != "0" -a "$DESTALLDN3" != "0" ]; then
            SOURCEALLDN="$( grep -w ALLDN $USERSOURCE/$USER )"
            if [ "$SOURCEALLDN" != "" ]; then
              SOURCEALLDN1="$( echo $SOURCEALLDN | awk '{print $2}' )"
              SOURCEALLDN2="$( echo $SOURCEALLDN | awk '{print $3}' )"
              SOURCEALLDN3="$( echo $SOURCEALLDN | awk '{print $4}' )"
              if [ "$SOURCEALLDN1" = "" ]; then
                SOURCEALLDN1="0"
              fi
              if [ "$SOURCEALLDN3 = "" ]; then
                SOURCEALLDN2="0"
              fi
              if [ "$SOURCEALLDN3 = "" ]; then
                SOURCEALLDN3="0"
              fi
              NEWALLDN1="$( expr "$SOURCEALLDN1" \+ "$DESTALLDN1" )"
              NEWALLDN2="$( expr "$SOURCEALLDN2" \+ "$DESTALLDN2" )"
              NEWALLDN3="$( expr "$SOURCEALLDN3" \+ "$DESTALLDN3" )"
              if [ "$NEWALLDN1" != "" -a "$NEWALLDN2" != "" -a "$NEWALLDN3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER ALLDN Stats $DESTNAME : $DESTALLDN1 $DESTALLDN2  $DESTALLDN3"
                  echo "$USER ALLDN Stats $SOURCENAME : $SOURCEALLDN1 $SOURCEALLDN2 $SOURCEALLDN3"
                  echo "$USER ALLDN Stats NEW : $NEWALLDN1 $NEWALLDN2 $NEWALLDN3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME ALLDN : $DESTALLDN1 $DESTALLDN2 $DESTALLDN3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME ALLDN : $SOURCEALLDN1 $SOURCEALLDN2 $SOURCEALLDN3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new ALLDN stats: $NEWALLDN1 $NEWALLDN2 $NEWALLDN3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^ALLDN.*/ALLDN $NEWALLDN1 $NEWALLDN2 $NEWALLDN3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^ALLDN.*/ALLDN $NEWALLDN1 $NEWALLDN2 $NEWALLDN3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^ALLDN.*/ALLDN 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^ALLDN.*/ALLDN 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new ALLDN stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read ALLDN from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching ALLDN on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read ALLDN from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## WKUP
    if [ "$MOVEWKUP" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTWKUP="$( grep -w WKUP $USERDEST/$USER )"
        if [ "$DESTWKUP" != "" ]; then
          DESTWKUP1="$( echo $DESTWKUP | awk '{print $2}' )"
          DESTWKUP2="$( echo $DESTWKUP | awk '{print $3}' )"
          DESTWKUP3="$( echo $DESTWKUP | awk '{print $4}' )"
          if [ "$DESTWKUP1" = "" ]; then
            DESTWKUP1="0"
          fi
          if [ "$DESTWKUP3 = "" ]; then
            DESTWKUP2="0"
          fi
          if [ "$DESTWKUP3 = "" ]; then
            DESTWKUP3="0"
          fi
          if [ "$DESTWKUP1" != "0" -a "$DESTWKUP2" != "0" -a "$DESTWKUP3" != "0" ]; then
            SOURCEWKUP="$( grep -w WKUP $USERSOURCE/$USER )"
            if [ "$SOURCEWKUP" != "" ]; then
              SOURCEWKUP1="$( echo $SOURCEWKUP | awk '{print $2}' )"
              SOURCEWKUP2="$( echo $SOURCEWKUP | awk '{print $3}' )"
              SOURCEWKUP3="$( echo $SOURCEWKUP | awk '{print $4}' )"
              if [ "$SOURCEWKUP1" = "" ]; then
                SOURCEWKUP1="0"
              fi
              if [ "$SOURCEWKUP3 = "" ]; then
                SOURCEWKUP2="0"
              fi
              if [ "$SOURCEWKUP3 = "" ]; then
                SOURCEWKUP3="0"
              fi
              NEWWKUP1="$( expr "$SOURCEWKUP1" \+ "$DESTWKUP1" )"
              NEWWKUP2="$( expr "$SOURCEWKUP2" \+ "$DESTWKUP2" )"
              NEWWKUP3="$( expr "$SOURCEWKUP3" \+ "$DESTWKUP3" )"
              if [ "$NEWWKUP1" != "" -a "$NEWWKUP2" != "" -a "$NEWWKUP3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER WKUP Stats $DESTNAME : $DESTWKUP1 $DESTWKUP2  $DESTWKUP3"
                  echo "$USER WKUP Stats $SOURCENAME : $SOURCEWKUP1 $SOURCEWKUP2 $SOURCEWKUP3"
                  echo "$USER WKUP Stats NEW : $NEWWKUP1 $NEWWKUP2 $NEWWKUP3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME WKUP : $DESTWKUP1 $DESTWKUP2 $DESTWKUP3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME WKUP : $SOURCEWKUP1 $SOURCEWKUP2 $SOURCEWKUP3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new WKUP stats: $NEWWKUP1 $NEWWKUP2 $NEWWKUP3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^WKUP.*/WKUP $NEWWKUP1 $NEWWKUP2 $NEWWKUP3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^WKUP.*/WKUP $NEWWKUP1 $NEWWKUP2 $NEWWKUP3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^WKUP.*/WKUP 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^WKUP.*/WKUP 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new WKUP stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read WKUP from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching WKUP on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read WKUP from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi


    ## WKDN
    if [ "$MOVEWKDN" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTWKDN="$( grep -w WKDN $USERDEST/$USER )"
        if [ "$DESTWKDN" != "" ]; then
          DESTWKDN1="$( echo $DESTWKDN | awk '{print $2}' )"
          DESTWKDN2="$( echo $DESTWKDN | awk '{print $3}' )"
          DESTWKDN3="$( echo $DESTWKDN | awk '{print $4}' )"
          if [ "$DESTWKDN1" = "" ]; then
            DESTWKDN1="0"
          fi
          if [ "$DESTWKDN3 = "" ]; then
            DESTWKDN2="0"
          fi
          if [ "$DESTWKDN3 = "" ]; then
            DESTWKDN3="0"
          fi
          if [ "$DESTWKDN1" != "0" -a "$DESTWKDN2" != "0" -a "$DESTWKDN3" != "0" ]; then
            SOURCEWKDN="$( grep -w WKDN $USERSOURCE/$USER )"
            if [ "$SOURCEWKDN" != "" ]; then
              SOURCEWKDN1="$( echo $SOURCEWKDN | awk '{print $2}' )"
              SOURCEWKDN2="$( echo $SOURCEWKDN | awk '{print $3}' )"
              SOURCEWKDN3="$( echo $SOURCEWKDN | awk '{print $4}' )"
              if [ "$SOURCEWKDN1" = "" ]; then
                SOURCEWKDN1="0"
              fi
              if [ "$SOURCEWKDN3 = "" ]; then
                SOURCEWKDN2="0"
              fi
              if [ "$SOURCEWKDN3 = "" ]; then
                SOURCEWKDN3="0"
              fi
              NEWWKDN1="$( expr "$SOURCEWKDN1" \+ "$DESTWKDN1" )"
              NEWWKDN2="$( expr "$SOURCEWKDN2" \+ "$DESTWKDN2" )"
              NEWWKDN3="$( expr "$SOURCEWKDN3" \+ "$DESTWKDN3" )"
              if [ "$NEWWKDN1" != "" -a "$NEWWKDN2" != "" -a "$NEWWKDN3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER WKDN Stats $DESTNAME : $DESTWKDN1 $DESTWKDN2  $DESTWKDN3"
                  echo "$USER WKDN Stats $SOURCENAME : $SOURCEWKDN1 $SOURCEWKDN2 $SOURCEWKDN3"
                  echo "$USER WKDN Stats NEW : $NEWWKDN1 $NEWWKDN2 $NEWWKDN3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME WKDN : $DESTWKDN1 $DESTWKDN2 $DESTWKDN3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME WKDN : $SOURCEWKDN1 $SOURCEWKDN2 $SOURCEWKDN3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new WKDN stats: $NEWWKDN1 $NEWWKDN2 $NEWWKDN3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^WKDN.*/WKDN $NEWWKDN1 $NEWWKDN2 $NEWWKDN3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^WKDN.*/WKDN $NEWWKDN1 $NEWWKDN2 $NEWWKDN3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^WKDN.*/WKDN 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^WKDN.*/WKDN 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new WKDN stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read WKDN from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching WKDN on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read WKDN from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## DAYUP
    if [ "$MOVEDAYUP" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTDAYUP="$( grep -w DAYUP $USERDEST/$USER )"
        if [ "$DESTDAYUP" != "" ]; then
          DESTDAYUP1="$( echo $DESTDAYUP | awk '{print $2}' )"
          DESTDAYUP2="$( echo $DESTDAYUP | awk '{print $3}' )"
          DESTDAYUP3="$( echo $DESTDAYUP | awk '{print $4}' )"
          if [ "$DESTDAYUP1" = "" ]; then
            DESTDAYUP1="0"
          fi
          if [ "$DESTDAYUP3 = "" ]; then
            DESTDAYUP2="0"
          fi
          if [ "$DESTDAYUP3 = "" ]; then
            DESTDAYUP3="0"
          fi
          if [ "$DESTDAYUP1" != "0" -a "$DESTDAYUP2" != "0" -a "$DESTDAYUP3" != "0" ]; then
            SOURCEDAYUP="$( grep -w DAYUP $USERSOURCE/$USER )"
            if [ "$SOURCEDAYUP" != "" ]; then
              SOURCEDAYUP1="$( echo $SOURCEDAYUP | awk '{print $2}' )"
              SOURCEDAYUP2="$( echo $SOURCEDAYUP | awk '{print $3}' )"
              SOURCEDAYUP3="$( echo $SOURCEDAYUP | awk '{print $4}' )"
              if [ "$SOURCEDAYUP1" = "" ]; then
                SOURCEDAYUP1="0"
              fi
              if [ "$SOURCEDAYUP3 = "" ]; then
                SOURCEDAYUP2="0"
              fi
              if [ "$SOURCEDAYUP3 = "" ]; then
                SOURCEDAYUP3="0"
              fi
              NEWDAYUP1="$( expr "$SOURCEDAYUP1" \+ "$DESTDAYUP1" )"
              NEWDAYUP2="$( expr "$SOURCEDAYUP2" \+ "$DESTDAYUP2" )"
              NEWDAYUP3="$( expr "$SOURCEDAYUP3" \+ "$DESTDAYUP3" )"
              if [ "$NEWDAYUP1" != "" -a "$NEWDAYUP2" != "" -a "$NEWDAYUP3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER DAYUP Stats $DESTNAME : $DESTDAYUP1 $DESTDAYUP2  $DESTDAYUP3"
                  echo "$USER DAYUP Stats $SOURCENAME : $SOURCEDAYUP1 $SOURCEDAYUP2 $SOURCEDAYUP3"
                  echo "$USER DAYUP Stats NEW : $NEWDAYUP1 $NEWDAYUP2 $NEWDAYUP3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME DAYUP : $DESTDAYUP1 $DESTDAYUP2 $DESTDAYUP3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME DAYUP : $SOURCEDAYUP1 $SOURCEDAYUP2 $SOURCEDAYUP3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new DAYUP stats: $NEWDAYUP1 $NEWDAYUP2 $NEWDAYUP3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^DAYUP.*/DAYUP $NEWDAYUP1 $NEWDAYUP2 $NEWDAYUP3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^DAYUP.*/DAYUP $NEWDAYUP1 $NEWDAYUP2 $NEWDAYUP3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^DAYUP.*/DAYUP 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^DAYUP.*/DAYUP 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new DAYUP stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read DAYUP from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching DAYUP on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read DAYUP from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## DAYDN
    if [ "$MOVEDAYDN" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTDAYDN="$( grep -w DAYDN $USERDEST/$USER )"
        if [ "$DESTDAYDN" != "" ]; then
          DESTDAYDN1="$( echo $DESTDAYDN | awk '{print $2}' )"
          DESTDAYDN2="$( echo $DESTDAYDN | awk '{print $3}' )"
          DESTDAYDN3="$( echo $DESTDAYDN | awk '{print $4}' )"
          if [ "$DESTDAYDN1" = "" ]; then
            DESTDAYDN1="0"
          fi
          if [ "$DESTDAYDN3 = "" ]; then
            DESTDAYDN2="0"
          fi
          if [ "$DESTDAYDN3 = "" ]; then
            DESTDAYDN3="0"
          fi
          if [ "$DESTDAYDN1" != "0" -a "$DESTDAYDN2" != "0" -a "$DESTDAYDN3" != "0" ]; then
            SOURCEDAYDN="$( grep -w DAYDN $USERSOURCE/$USER )"
            if [ "$SOURCEDAYDN" != "" ]; then
              SOURCEDAYDN1="$( echo $SOURCEDAYDN | awk '{print $2}' )"
              SOURCEDAYDN2="$( echo $SOURCEDAYDN | awk '{print $3}' )"
              SOURCEDAYDN3="$( echo $SOURCEDAYDN | awk '{print $4}' )"
              if [ "$SOURCEDAYDN1" = "" ]; then
                SOURCEDAYDN1="0"
              fi
              if [ "$SOURCEDAYDN3 = "" ]; then
                SOURCEDAYDN2="0"
              fi
              if [ "$SOURCEDAYDN3 = "" ]; then
                SOURCEDAYDN3="0"
              fi
              NEWDAYDN1="$( expr "$SOURCEDAYDN1" \+ "$DESTDAYDN1" )"
              NEWDAYDN2="$( expr "$SOURCEDAYDN2" \+ "$DESTDAYDN2" )"
              NEWDAYDN3="$( expr "$SOURCEDAYDN3" \+ "$DESTDAYDN3" )"
              if [ "$NEWDAYDN1" != "" -a "$NEWDAYDN2" != "" -a "$NEWDAYDN3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER DAYDN Stats $DESTNAME : $DESTDAYDN1 $DESTDAYDN2  $DESTDAYDN3"
                  echo "$USER DAYDN Stats $SOURCENAME : $SOURCEDAYDN1 $SOURCEDAYDN2 $SOURCEDAYDN3"
                  echo "$USER DAYDN Stats NEW : $NEWDAYDN1 $NEWDAYDN2 $NEWDAYDN3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME DAYDN : $DESTDAYDN1 $DESTDAYDN2 $DESTDAYDN3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME DAYDN : $SOURCEDAYDN1 $SOURCEDAYDN2 $SOURCEDAYDN3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new DAYDN stats: $NEWDAYDN1 $NEWDAYDN2 $NEWDAYDN3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^DAYDN.*/DAYDN $NEWDAYDN1 $NEWDAYDN2 $NEWDAYDN3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^DAYDN.*/DAYDN $NEWDAYDN1 $NEWDAYDN2 $NEWDAYDN3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^DAYDN.*/DAYDN 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^DAYDN.*/DAYDN 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new DAYDN stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read DAYDN from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching DAYDN on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read DAYDN from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## MONTHUP
    if [ "$MOVEMONTHUP" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTMONTHUP="$( grep -w MONTHUP $USERDEST/$USER )"
        if [ "$DESTMONTHUP" != "" ]; then
          DESTMONTHUP1="$( echo $DESTMONTHUP | awk '{print $2}' )"
          DESTMONTHUP2="$( echo $DESTMONTHUP | awk '{print $3}' )"
          DESTMONTHUP3="$( echo $DESTMONTHUP | awk '{print $4}' )"
          if [ "$DESTMONTHUP1" = "" ]; then
            DESTMONTHUP1="0"
          fi
          if [ "$DESTMONTHUP3 = "" ]; then
            DESTMONTHUP2="0"
          fi
          if [ "$DESTMONTHUP3 = "" ]; then
            DESTMONTHUP3="0"
          fi
          if [ "$DESTMONTHUP1" != "0" -a "$DESTMONTHUP2" != "0" -a "$DESTMONTHUP3" != "0" ]; then
            SOURCEMONTHUP="$( grep -w MONTHUP $USERSOURCE/$USER )"
            if [ "$SOURCEMONTHUP" != "" ]; then
              SOURCEMONTHUP1="$( echo $SOURCEMONTHUP | awk '{print $2}' )"
              SOURCEMONTHUP2="$( echo $SOURCEMONTHUP | awk '{print $3}' )"
              SOURCEMONTHUP3="$( echo $SOURCEMONTHUP | awk '{print $4}' )"
              if [ "$SOURCEMONTHUP1" = "" ]; then
                SOURCEMONTHUP1="0"
              fi
              if [ "$SOURCEMONTHUP3 = "" ]; then
                SOURCEMONTHUP2="0"
              fi
              if [ "$SOURCEMONTHUP3 = "" ]; then
                SOURCEMONTHUP3="0"
              fi
              NEWMONTHUP1="$( expr "$SOURCEMONTHUP1" \+ "$DESTMONTHUP1" )"
              NEWMONTHUP2="$( expr "$SOURCEMONTHUP2" \+ "$DESTMONTHUP2" )"
              NEWMONTHUP3="$( expr "$SOURCEMONTHUP3" \+ "$DESTMONTHUP3" )"
              if [ "$NEWMONTHUP1" != "" -a "$NEWMONTHUP2" != "" -a "$NEWMONTHUP3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER MONTHUP Stats $DESTNAME : $DESTMONTHUP1 $DESTMONTHUP2  $DESTMONTHUP3"
                  echo "$USER MONTHUP Stats $SOURCENAME : $SOURCEMONTHUP1 $SOURCEMONTHUP2 $SOURCEMONTHUP3"
                  echo "$USER MONTHUP Stats NEW : $NEWMONTHUP1 $NEWMONTHUP2 $NEWMONTHUP3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME MONTHUP : $DESTMONTHUP1 $DESTMONTHUP2 $DESTMONTHUP3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME MONTHUP : $SOURCEMONTHUP1 $SOURCEMONTHUP2 $SOURCEMONTHUP3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new MONTHUP stats: $NEWMONTHUP1 $NEWMONTHUP2 $NEWMONTHUP3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^MONTHUP.*/MONTHUP $NEWMONTHUP1 $NEWMONTHUP2 $NEWMONTHUP3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^MONTHUP.*/MONTHUP $NEWMONTHUP1 $NEWMONTHUP2 $NEWMONTHUP3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^MONTHUP.*/MONTHUP 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^MONTHUP.*/MONTHUP 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new MONTHUP stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read MONTHUP from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching MONTHUP on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read MONTHUP from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## MONTHDN
    if [ "$MOVEMONTHDN" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTMONTHDN="$( grep -w MONTHDN $USERDEST/$USER )"
        if [ "$DESTMONTHDN" != "" ]; then
          DESTMONTHDN1="$( echo $DESTMONTHDN | awk '{print $2}' )"
          DESTMONTHDN2="$( echo $DESTMONTHDN | awk '{print $3}' )"
          DESTMONTHDN3="$( echo $DESTMONTHDN | awk '{print $4}' )"
          if [ "$DESTMONTHDN1" = "" ]; then
            DESTMONTHDN1="0"
          fi
          if [ "$DESTMONTHDN3 = "" ]; then
            DESTMONTHDN2="0"
          fi
          if [ "$DESTMONTHDN3 = "" ]; then
            DESTMONTHDN3="0"
          fi
          if [ "$DESTMONTHDN1" != "0" -a "$DESTMONTHDN2" != "0" -a "$DESTMONTHDN3" != "0" ]; then
            SOURCEMONTHDN="$( grep -w MONTHDN $USERSOURCE/$USER )"
            if [ "$SOURCEMONTHDN" != "" ]; then
              SOURCEMONTHDN1="$( echo $SOURCEMONTHDN | awk '{print $2}' )"
              SOURCEMONTHDN2="$( echo $SOURCEMONTHDN | awk '{print $3}' )"
              SOURCEMONTHDN3="$( echo $SOURCEMONTHDN | awk '{print $4}' )"
              if [ "$SOURCEMONTHDN1" = "" ]; then
                SOURCEMONTHDN1="0"
              fi
              if [ "$SOURCEMONTHDN3 = "" ]; then
                SOURCEMONTHDN2="0"
              fi
              if [ "$SOURCEMONTHDN3 = "" ]; then
                SOURCEMONTHDN3="0"
              fi
              NEWMONTHDN1="$( expr "$SOURCEMONTHDN1" \+ "$DESTMONTHDN1" )"
              NEWMONTHDN2="$( expr "$SOURCEMONTHDN2" \+ "$DESTMONTHDN2" )"
              NEWMONTHDN3="$( expr "$SOURCEMONTHDN3" \+ "$DESTMONTHDN3" )"
              if [ "$NEWMONTHDN1" != "" -a "$NEWMONTHDN2" != "" -a "$NEWMONTHDN3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER MONTHDN Stats $DESTNAME : $DESTMONTHDN1 $DESTMONTHDN2  $DESTMONTHDN3"
                  echo "$USER MONTHDN Stats $SOURCENAME : $SOURCEMONTHDN1 $SOURCEMONTHDN2 $SOURCEMONTHDN3"
                  echo "$USER MONTHDN Stats NEW : $NEWMONTHDN1 $NEWMONTHDN2 $NEWMONTHDN3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME MONTHDN : $DESTMONTHDN1 $DESTMONTHDN2 $DESTMONTHDN3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME MONTHDN : $SOURCEMONTHDN1 $SOURCEMONTHDN2 $SOURCEMONTHDN3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new MONTHDN stats: $NEWMONTHDN1 $NEWMONTHDN2 $NEWMONTHDN3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^MONTHDN.*/MONTHDN $NEWMONTHDN1 $NEWMONTHDN2 $NEWMONTHDN3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^MONTHDN.*/MONTHDN $NEWMONTHDN1 $NEWMONTHDN2 $NEWMONTHDN3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^MONTHDN.*/MONTHDN 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^MONTHDN.*/MONTHDN 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new MONTHDN stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read MONTHDN from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching MONTHDN on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read MONTHDN from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi

    ## NUKE
    if [ "$MOVENUKE" = "TRUE" -a "$SKIP" != "YES" ]; then
      if [ -w "$USERSOURCE/$USER" ]; then
        DESTNUKE="$( grep -w NUKE $USERDEST/$USER )"
        if [ "$DESTNUKE" != "" ]; then
          DESTNUKE1="$( echo $DESTNUKE | awk '{print $2}' )"
          DESTNUKE2="$( echo $DESTNUKE | awk '{print $3}' )"
          DESTNUKE3="$( echo $DESTNUKE | awk '{print $4}' )"
          if [ "$DESTNUKE1" = "" ]; then
            DESTNUKE1="0"
          fi
          if [ "$DESTNUKE3 = "" ]; then
            DESTNUKE2="0"
          fi
          if [ "$DESTNUKE3 = "" ]; then
            DESTNUKE3="0"
          fi
          if [ "$DESTNUKE1" != "0" -a "$DESTNUKE2" != "0" -a "$DESTNUKE3" != "0" ]; then
            SOURCENUKE="$( grep -w NUKE $USERSOURCE/$USER )"
            if [ "$SOURCENUKE" != "" ]; then
              SOURCENUKE1="$( echo $SOURCENUKE | awk '{print $2}' )"
              SOURCENUKE2="$( echo $SOURCENUKE | awk '{print $3}' )"
              SOURCENUKE3="$( echo $SOURCENUKE | awk '{print $4}' )"
              if [ "$SOURCENUKE1" = "" ]; then
                SOURCENUKE1="0"
              fi
              if [ "$SOURCENUKE3 = "" ]; then
                SOURCENUKE2="0"
              fi
              if [ "$SOURCENUKE3 = "" ]; then
                SOURCENUKE3="0"
              fi
              NEWNUKE1="$( expr "$SOURCENUKE1" \+ "$DESTNUKE1" )"
              NEWNUKE2="$( expr "$SOURCENUKE2" \+ "$DESTNUKE2" )"
              NEWNUKE3="$( expr "$SOURCENUKE3" \+ "$DESTNUKE3" )"
              if [ "$NEWNUKE1" != "" -a "$NEWNUKE2" != "" -a "$NEWNUKE3" != "" ]; then
                if [ "$DEBUG" = "TRUE" ]; then
                  echo "$USER NUKE Stats $DESTNAME : $DESTNUKE1 $DESTNUKE2  $DESTNUKE3"
                  echo "$USER NUKE Stats $SOURCENAME : $SOURCENUKE1 $SOURCENUKE2 $SOURCENUKE3"
                  echo "$USER NUKE Stats NEW : $NEWNUKE1 $NEWNUKE2 $NEWNUKE3"
                fi
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $DESTNAME NUKE : $DESTNUKE1 $DESTNUKE2 $DESTNUKE3\" >> $STATLOG
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER $SOURCENAME NUKE : $SOURCENUKE1 $SOURCENUKE2 $SOURCENUKE3\" >> $STATLOG
                fi
                if [ "$STATLOG" != "" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNC: \"$USER got new NUKE stats: $NEWNUKE1 $NEWNUKE2 $NEWNUKE3\" >> $STATLOG
                fi

                if [ -r $USERSOURCE/$USER ]; then
                  if [ -r $USERDEST/$USER ]; then
                    if [ -e $TMP/TEMPSTATS.$USER.NEW ]; then
                      sed -e "s/^NUKE.*/NUKE $NEWNUKE1 $NEWNUKE2 $NEWNUKE3/" $TMP/TEMPSTATS.$USER.NEW > $TMP/TEMPSTATS.$USER.1
                      mv -f $TMP/TEMPSTATS.$USER.1 $TMP/TEMPSTATS.$USER.NEW
                    else
                      sed -e "s/^NUKE.*/NUKE $NEWNUKE1 $NEWNUKE2 $NEWNUKE3/" $USERSOURCE/$USER > $TMP/TEMPSTATS.$USER.NEW
                    fi
                    if [ -e $TMP/TEMPSTATS.$USER.DEST.NEW ]; then
                      sed -e "s/^NUKE.*/NUKE 0 0 0/" $TMP/TEMPSTATS.$USER.DEST.NEW > $TMP/TEMPSTATS.$USER.DEST.1
                      mv -f $TMP/TEMPSTATS.$USER.DEST.1 $TMP/TEMPSTATS.$USER.DEST.NEW
                    else
                      sed -e "s/^NUKE.*/NUKE 0 0 0/" $USERDEST/$USER > $TMP/TEMPSTATS.$USER.DEST.NEW
                    fi
                    sleep 0.1
                    NEWEXISTS="TRUE"
                  else
                    if [ "$HIGHDEBUG" = "TRUE" ]; then
                      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERDEST/$USER\" >> $STATLOG
                    fi
                  fi
                else
                  if [ "$HIGHDEBUG" = "TRUE" ]; then
                    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read from to $USERSOURCE/$USER\" >> $STATLOG
                  fi
                fi
              else
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Calculating new NUKE stats for $USER failed!\" >> $STATLOG
                fi
              fi
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read NUKE from $USERSOURCE/$USER\" >> $STATLOG
              fi
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Not touching NUKE on $USER - Some stats are empty or 0\" >> $STATLOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not read NUKE from $USERDEST/$USER\" >> $STATLOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERSOURCE/$USER\" >> $STATLOG
        fi
      fi
    fi


    if [ "$NEWEXISTS" = "TRUE" ]; then
      if [ "$USERSOURCEORG" != "" ]; then
        if [ -w "$USERSOURCEORG/$USER" ]; then
          SOURCEOK="YES"
        fi
      else
        if [ -w "$USERSOURCE/$USER" ]; then
          SOURCEOK="YES"
        fi
      fi
      if [ -w "$USERDEST/$USER" ]; then
        if [ "$SOURCEOK" = "YES" ]; then
          if [ "$USERSOURCEORG" != "" ]; then
            mv -f $TMP/TEMPSTATS.$USER.NEW $USERSOURCEORG/$USER
          else
            mv -f $TMP/TEMPSTATS.$USER.NEW $USERSOURCE/$USER
          fi
          mv -f $TMP/TEMPSTATS.$USER.DEST.NEW $USERDEST/$USER
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to source file $USER\" >> $STATLOG
          fi
          if [ "$DEBUG" = "TRUE" ]; then
            echo "Error: Can not write sourcefile $USER - No stats moved for this user."
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Can not write to $USERDEST/$USER\" >> $STATLOG
        fi
        if [ "$DEBUG" = "TRUE" ]; then
          echo "Error: Can not write to $USERDEST/$USER - No stats moved for this user."
        fi
      fi
      SOURCEOK="NO"
      NEWEXISTS="FALSE"
    fi

    SKIP="NO"
    if [ "$SLOWTIME" != "" ]; then
      sleep $SLOWTIME
    fi
  done
  rm -f $LOCKFILE
  exit 0
fi  


######## SYNC ################################################################33
## Check if group is up to date.
if [ -w $PASSWDDEST/group ]; then
  destsum="$( $MD5BIN $PASSWDDEST/group | awk '{print $1}' )"
  sourcesum="$( $MD5BIN $PASSWDSOURCE/group | awk '{print $1}' )"
  if [ "$destsum" != "$sourcesum" ]; then
    cp -f $PASSWDSOURCE/group $PASSWDDEST/group
    if [ "$DEBUG" = "TRUE" ]; then
      echo "group file synced with source"
    fi
    if [ "$LOG" != "" ]; then
      echo `date "+%a %b %e %T %Y"` USYNC: \"group file synced with source\" >> $LOG
    fi
  fi
else
  cp -f $PASSWDSOURCE/group $PASSWDDEST/group
  if [ "$DEBUG" = "TRUE" ]; then
    echo "group file non existing on destination. Grabbing it"
  fi
  if [ "$LOG" != "" ]; then
    echo `date "+%a %b %e %T %Y"` USYNC: \"group file non existing on destination. Grabbing it\" >> $LOG
  fi
fi

if [ -w $PASSWDDEST/passwd ]; then
  ## Check if passwd is up to date
  destsum="$( $MD5BIN $PASSWDDEST/passwd | awk '{print $1}' )"
  sourcesum="$( $MD5BIN $PASSWDSOURCE/passwd | awk '{print $1}' )"
  if [ "$destsum" != "$sourcesum" ]; then
    cp -f $PASSWDSOURCE/passwd $PASSWDDEST/passwd
    if [ "$DEBUG" = "TRUE" ]; then
      echo "passwd file synced with source"
    fi
    if [ "$LOG" != "" ]; then
      echo `date "+%a %b %e %T %Y"` USYNC: \"passwd file synced with source\" >> $LOG
    fi
  fi
else
  cp -f $PASSWDSOURCE/passwd $PASSWDDEST/passwd
  if [ "$DEBUG" = "TRUE" ]; then
    echo "passwd file non existing on destination. Grabbing it"
  fi
  if [ "$LOG" != "" ]; then
    echo `date "+%a %b %e %T %Y"` USYNC: \"passwd file non existing on destination. Grabbing it.\" >> $LOG
  fi
fi

######## CHANGE ##########################################################

## User changes, based on source
cd $USERSOURCE

for USER in `grep -w FLAGS * | awk '{print $1}' | awk -F":" '{print $1}' | egrep -v $EXCLUDE`; do

  EXISTS=""
  if [ -w $USERDEST/$USER ]; then
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Syncing existing user: $USER\" >> $LOG
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Syncing: $USER"
    fi
    EXISTS="YES"
  else
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Syncing NEW user $USER\" >> $LOG
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "$USER does not exist"
    fi
    EXISTS="NO"
  fi

  size="$( du -b $USER | awk '{print $1}' )"
  if [ "$size" != "4096" ]; then
    if [ "$size" = "" ]; then
      size="ErrorReading"
    fi
    if [ "$DEBUG" = "TRUE" ]; then
      echo "$USER does not look like a userfile, or its corrupt on source. Size: $size"
    fi
    if [ "$LOG" != "" ]; then
      echo `date "+%a %b %e %T %Y"` USYNC: \"Error: $USER does not look like a userfile. Size should be 4096. Reported:$size\" >> $LOG
    fi
    SKIP="YES"
  fi

  ## Check if he was previously deleted from destination server.
  ## If so, restore that file instead of resetting stats.
  if [ "$BACKUPDEST" != "" ]; then
    if [ -e "$BACKUPDEST/deleted/$USER" ]; then
      mv -f $BACKUPDEST/deleted/$USER $USERDEST/$USER
      SKIPRES="YES"
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Found previosly deleted user: $USER - Restored"
      fi
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Found previously deleted user: $USER - Restored\" >> $LOG
      fi
    fi
  fi

  if [ "$EXISTS" = "NO" -a "$SKIP" != "YES" -a "$SKIPRES" != "YES" ]; then
    VERIFY=""
    VERIFY="$( grep -w TAGLINE $USERSOURCE/$USER | egrep -w 'WKDN|ALLDN|WKUP|WKDN|DAYUP|DAYDN|MONTHUP|MONTHDN|NUKE|TIME' )"
    if [ "$VERIFY" != "" ]; then
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Warning: TAGLINE of $USER contains a bad word. This can cause probs. Not adding/syncing."
      fi
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Warning: TAGLINE of $USER contains a bad word. Not adding/syncing\" >> $LOG
        echo `date "+%a %b %e %T %Y"` USYNC: \"Warning: Bad words are WKDN ALLDN WKUP WKDN DAYUP DAYDN MONTHUP MONTHDN NUKE TIME\" >> $LOG
      fi
      SKIP="YES"
    fi
  fi

  ## User did not exist before
  if [ "$EXISTS" = "NO" -a "$SKIP" != "YES" -a "$SKIPRES" != "YES" ]; then

    ## Do a clean copy and remove stats/creds
    cp -f $USER $USERDEST/$USER
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Resetting all stats on new user: $USER"
    fi
    if [ "$LOG" != "" ]; then
      echo `date "+%a %b %e %T %Y"` USYNC: \"New user added: $USER\" >> $LOG
    fi

    ## Reset CREDITS
    sed -e "s/^CREDITS.*/CREDITS 0/" $USERDEST/$USER > $TMP/TEMP.$USER.CREDITS
    mv -f $TMP/TEMP.$USER.CREDITS $USERDEST/$USER

    ## Reset ALL UPLOAD stats
    sed -e "s/^ALLUP.*/ALLUP 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.ALLUP
    mv -f $TMP/TEMP.$USER.ALLUP $USERDEST/$USER
 
    ## Reset ALL DOWNLOAD stats
    sed -e "s/^ALLDN.*/ALLDN 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.ALLDN
    mv -f $TMP/TEMP.$USER.ALLDN $USERDEST/$USER

    ## Reset WEEK UP stats
    sed -e "s/^WKUP.*/WKUP 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.WKUP
    mv -f $TMP/TEMP.$USER.WKUP $USERDEST/$USER

    ## Reset WEEK DOWNLOAD stats
    sed -e "s/^WKDN.*/WKDN 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.WKDN
    mv -f $TMP/TEMP.$USER.WKDN $USERDEST/$USER

    ## Reset DAY UPLOAD stats
    sed -e "s/^DAYUP.*/DAYUP 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.DAYUP
    mv -f $TMP/TEMP.$USER.DAYUP $USERDEST/$USER

    ## Reset DAY DOWNLOAD stats
    sed -e "s/^DAYDN.*/DAYDN 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.DAYDN
    mv -f $TMP/TEMP.$USER.DAYDN $USERDEST/$USER

    ## Reset MONTH UPLOAD stats
    sed -e "s/^MONTHUP.*/MONTHUP 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.MONTHUP
    mv -f $TMP/TEMP.$USER.MONTHUP $USERDEST/$USER

    ## Reset MONTH DOWNLOAD stats
    sed -e "s/^MONTHDN.*/MONTHDN 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.MONTHDN
    mv -f $TMP/TEMP.$USER.MONTHDN $USERDEST/$USER

    ## Reset NUKE counter
    sed -e "s/^NUKE.*/NUKE 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.NUKE
    mv -f $TMP/TEMP.$USER.NUKE $USERDEST/$USER

    ## Reset TIME online
    sed -e "s/^TIME.*/TIME 0 0 0 0/" $USERDEST/$USER > $TMP/TEMP.$USER.TIME
    mv -f $TMP/TEMP.$USER.TIME $USERDEST/$USER

    ## Reset NUKE counter
    sed -e "s/^SLOTS.*/SLOTS 0 0 /" $USERDEST/$USER > $TMP/TEMP.$USER.SLOTS
    mv -f $TMP/TEMP.$USER.SLOTS $USERDEST/$USER

    if [ "$PERMS" != "" ]; then
      chmod $PERMS $USERDEST/$USER >/dev/null 2>&1
    fi

  fi
  SKIPRES="NO"

  ## User already existed. Lets sync the ips/groups etc
  if [ "$EXISTS" = "YES" -a "$SKIP" != "YES" ]; then

    ## Sync FLAGS
    DESTFLAGS="$( grep -w FLAGS $USERDEST/$USER | awk '{print $2}' )"
    if [ -r $USERSOURCE/$USER ]; then
      if [ "$NOGADMINS" = "TRUE" ]; then
        SOURCEFLAGS="$( grep -w FLAGS $USERSOURCE/$USER | awk '{print $2}' | tr -d '2' )"
      else
        SOURCEFLAGS="$( grep -w FLAGS $USERSOURCE/$USER | awk '{print $2}' )"
      fi
    else
      SOURCEFLAGS=""
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read FLAGS from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
    if [ "$SOURCEFLAGS" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - FLAGS = $DESTNAME $DESTFLAGS / $SOURCENAME $SOURCEFLAGS\" >> $LOG
      fi
      if [ "$DESTFLAGS" != "$SOURCEFLAGS" ]; then
        sed -e "s/^FLAGS.*/FLAGS $SOURCEFLAGS/" $USERDEST/$USER > $TMP/TEMP.$USER.FLAGS
        mv -f $TMP/TEMP.$USER.FLAGS $USERDEST/$USER
        if [ "$DEBUG" = "TRUE" ]; then
          echo "$USER - FLAGS changed from $DESTFLAGS to $SOURCEFLAGS"
        fi
        if [ "$LOG" != "" ]; then
          echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - FLAGS change from $DESTFLAGS to $SOURCEFLAGS\" >> $LOG
        fi
      fi
    fi

    ## Sync RATIO
    DESTRATIO="$( grep -w RATIO $USERDEST/$USER | awk '{print $2}' )"
    if [ -r $USERSOURCE/$USER ]; then
      SOURCERATIO="$( grep -w RATIO $USERSOURCE/$USER | awk '{print $2}' )"
    else
      SOURCERATIO=""
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read RATIO from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
    if [ "$SOURCERATIO" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - RATIO = $DESTNAME $DESTRATIO / $SOURCENAME $SOURCERATIO\" >> $LOG
      fi
      if [ "$DESTRATIO" != "$SOURCERATIO" ]; then
        sed -e "s/^RATIO.*/RATIO $SOURCERATIO/" $USERDEST/$USER > $TMP/TEMP.$USER.RATIO
        mv -f $TMP/TEMP.$USER.RATIO $USERDEST/$USER
        if [ "$DEBUG" = "TRUE" ]; then
          echo "$USER - RATIO changed from $DESTRATIO to $SOURCERATIO"
        fi
        if [ "$LOG" != "" ]; then
          echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - RATIO change from $DESTRATIO to $SOURCERATIO\" >> $LOG
        fi
      fi
    fi

    ## Sync GENERAL
    DESTGENERAL="$( grep -w GENERAL $USERDEST/$USER )"
    if [ -r $USERSOURCE/$USER ]; then
      SOURCEGENERAL="$( grep -w GENERAL $USERSOURCE/$USER )"
    else
      SOURCEGENERAL=""
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read GENERAL from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
    if [ "$SOURCEGENERAL" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - GENERAL = $DESTNAME $DESTGENERAL / $SOURCENAME $SOURCEGENERAL\" >> $LOG
      fi

      if [ "$SHAREALLOTMENT" = "FALSE" ]; then
        ALLOTMENTNR="$( echo "$SOURCEGENERAL" | awk -F"," '{print $2}' | awk '{print $1}' )"
        if [ "$ALLOTMENTNR" != "0" ]; then
          ALLOTMENT="YES"
        else
          ALLOTMENT="NO"
        fi
        if [ "$ALLOTMENT" = "YES" ]; then
          SOURCEGENFIRST="$( echo "$SOURCEGENERAL" | awk -F"," '{print $1}' | awk '{print $2}' )"
          SOURCEGENTHIRD="$( echo "$SOURCEGENERAL" | awk '{print $3}' )"
          SOURCEGENFOURTH="$( echo "$SOURCEGENERAL" | awk '{print $4}' )"
	  SOURCEGENFIFTH="$( echo "$SOURCEGENERAL" | awk '{print $5}' )"
	  SOURCEGENERAL="GENERAL $SOURCEGENFIRST,1000 $SOURCEGENTHIRD $SOURCEGENFOURTH $SOURCEGENFIFTH"
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USER have $ALLOTMENTNR Kb allotment on $SOURCENAME - Setting to 1000 Kb on $DESTNAME"
          fi
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER have $ALLOTMENTNR allotment on $SOURCENAME - Setting to 1000 kb.\" >> $LOG
          fi
        fi
      fi

      if [ "$DESTGENERAL" != "$SOURCEGENERAL" ]; then
        ## Sync general from source
        cat $USERDEST/$USER | grep -v "GENERAL " > $TMP/TEMP.$USER.GENERAL
        echo "$SOURCEGENERAL" >> $TMP/TEMP.$USER.GENERAL
        mv -f $TMP/TEMP.$USER.GENERAL $USERDEST/$USER
        if [ "$DEBUG" = "TRUE" ]; then
          echo "$USER - GENERAL changed from $DESTGENERAL to $SOURCEGENERAL"
        fi
        if [ "$LOG" != "" ]; then
          echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - GENERAL change from $DESTGENERAL to $SOURCEGENERAL\" >> $LOG
        fi
      fi
    fi
    DESTGENERAL=""
    SOURCEGENERAL=""

    ## Sync LOGINS
    DESTLOGINS="$( grep -w LOGINS $USERDEST/$USER )"
    if [ -r $USERSOURCE/$USER ]; then
      SOURCELOGINS="$( grep -w LOGINS $USERSOURCE/$USER )"
    else
      SOURCELOGINS=""
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read LOGINS from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
    if [ "$SOURCELOGIN" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - LOGINS = $DESTNAME $DESTLOGINS / $SOURCENAME $SOURCELOGINS\" >> $LOG
      fi
      if [ "$DESTLOGIN" != "$SOURCELOGIN" ]; then
        ## Sync login from source
        cat $USERDEST/$USER | grep -v "LOGINS " > $TMP/TEMP.$USER.LOGINS
        echo "$SOURCELOGINS" >> $TMP/TEMP.$USER.LOGINS
        mv -f $TMP/TEMP.$USER.LOGINS $USERDEST/$USER
        if [ "$DEBUG" = "TRUE" ]; then
          echo "$USER - LOGINS changed from $DESTLOGINS to $SOURCELOGINS"
        fi
        if [ "$LOG" != "" ]; then
          echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - LOGINS change from $DESTLOGINS to $SOURCEOGINS\" >> $LOG
        fi
      fi
    fi
    DESTLOGINS=""
    SOURCELOGINS=""

    ## Sync TAGLINE
    DESTTAG="$( grep -w TAGLINE $USERDEST/$USER )"
    if [ -r $USERSOURCE/$USER ]; then
      SOURCETAG="$( grep -w TAGLINE $USERSOURCE/$USER )"
    else
      SOURCETAG=""
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read TAGLINE from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
    if [ "$SOURCETAG" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - TAGLINE = $DESTNAME $DESTTAG / $SOURCENAME $SOURCETAG\" >> $LOG
      fi
      if [ "$DESTTAG" != "$SOURCETAG" ]; then
        ## Sync tagline from source
        cat $USERDEST/$USER | grep -v "TAGLINE " > $TMP/TEMP.$USER.TAGLINE
        echo "$SOURCETAG" >> $TMP/TEMP.$USER.TAGLINE
        mv -f $TMP/TEMP.$USER.TAGLINE $USERDEST/$USER
        if [ "$DEBUG" = "TRUE" ]; then
          echo "$USER - TAGLINE changed from $DESTTAG to $SOURCETAG"
        fi
        if [ "$LOG" != "" ]; then
          echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - TAGLINE change from $DESTTAG to $SOURCETAG\" >> $LOG
        fi
      fi
    fi
    DESTTAG=""
    SOURCETAG=""

    ## Sync Primary group.
    DESTPRI="$( grep -w GROUP $USERDEST/$USER | head -n1 )"
    if [ -r $USERSOURCE/$USER ]; then
      SOURCEPRI="$( grep -w GROUP $USERSOURCE/$USER | head -n1 )"
    else
      SOURCEPRI=""
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read PRIGroup from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
    if [ "$SOURCEPRI" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - PRIGRoup = $DESTNAME $DESTPRI / $SOURCENAME $SOURCEPRI\" >> $LOG
      fi
      if [ "$DESTPRI" != "$SOURCEPRI" ]; then
        ## Remove pri group if its elsewhere.
        grep -w -v -F -- "$SOURCEPRI" $USERDEST/$USER > $TMP/TEMP.$USER.PRIGROUP1
        sed -e "s/^$DESTPRI/$SOURCEPRI/" $TMP/TEMP.$USER.PRIGROUP1 > $TMP/TEMP.$USER.PRIGROUP
        mv -f $TMP/TEMP.$USER.PRIGROUP $USERDEST/$USER
        rm -f $TMP/TEMP.$USER.PRIGROUP1
        if [ "$DEBUG" = "TRUE" ]; then
          echo "$USER - Primary group changed to $SOURCEPRI"
        fi
        if [ "$LOG" != "" ]; then
          echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - Primary group changed to $SOURCEPRI\" >> $LOG
        fi
      fi
    fi

    ## Add new groups.
    if [ -r $USERSOURCE/$USER ]; then
      for group in `grep -w GROUP $USERSOURCE/$USER | awk '{print $2}'`; do
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Checking GROUP $group\" >> $LOG
        fi
        CHECK="$( grep -w GROUP $USERDEST/$USER | grep -w $group )"
        if [ "$CHECK" = "" ]; then
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USER - added GROUP $group"
          fi
          if [ "$LOG" != "" ]; then
            echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - added GROUP $group\" >> $LOG
          fi
          echo "GROUP $group" >> $USERDEST/$USER
        fi
        CHECK=""
      done
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read GROUP from $USERSOURCE/$USER\" >> $LOG
      fi
    fi

    ## Removing redundant groups
    if [ -r $USERSOURCE/$USER ]; then
      for group in `grep -w GROUP $USERDEST/$USER | awk '{print $2}'`; do
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Checking GROUP $group -redundant groups-\" >> $LOG
        fi
        CHECK="$( grep -w GROUP $USERSOURCE/$USER | grep -w $group )"
        if [ "$CHECK" = "" ]; then
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USER - removing GROUP $group"
          fi
          if [ "$LOG" != "" ]; then
            echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - removing GROUP $group\" >> $LOG
          fi
          grep -v "GROUP $group" $USERDEST/$USER > $TMP/TEMP.$USER.GRPDEL
          mv -f $TMP/TEMP.$USER.GRPDEL $USERDEST/$USER
        fi
        CHECK=""
      done
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read GROUP from $USERSOURCE/$USER -redundant groups-\" >> $LOG
      fi
    fi

    ## Add new private groups.
    if [ -r $USERSOURCE/$USER ]; then
      for privgroup in `grep -w PRIVATE $USERSOURCE/$USER | awk '{print $2}'`; do
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Checking PRIVATE group $privgroup\" >> $LOG
        fi
        CHECK="$( grep "PRIVATE " $USERDEST/$USER | grep -w $privgroup )"
        if [ "$CHECK" = "" ]; then
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USER - added PRIVATE GROUP $privgroup"
          fi
          if [ "$LOG" != "" ]; then
            echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - added PRIVATE GROUP $privgroup\" >> $LOG
          fi
          echo "PRIVATE $privgroup" >> $USERDEST/$USER
        fi
        CHECK=""
      done
      privgroup=""
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read PRIVAE groups from $USERSOURCE/$USER\" >> $LOG
      fi
    fi

    ## Removing redundant private groups
    if [ -r $USERSOURCE/$USER ]; then
      for privgroup in `grep -w PRIVATE $USERDEST/$USER | awk '{print $2}'`; do
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Checking PRIVATE group $privgroup -redundant groups-\" >> $LOG
        fi
        CHECK="$( grep "PRIVATE " $USERSOURCE/$USER | grep -w $privgroup )"
        if [ "$CHECK" = "" ]; then
          if [ "$DEBUG" = "TRUE" ]; then
            echo "$USER - removing PRIVATE GROUP $privgroup"
          fi
          if [ "$LOG" != "" ]; then
            echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - removing PRIVATE GROUP $privgroup\" >> $LOG
          fi
          grep -v "PRIVATE $privgroup" $USERDEST/$USER > $TMP/TEMP.$USER.GRPDEL
          mv -f $TMP/TEMP.$USER.GRPDEL $USERDEST/$USER
        fi
        CHECK=""
      done
      privgroup=""
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read PRIVATE group from $USERSOURCE/$USER -redundant groups-\" >> $LOG
      fi
    fi

    ## Add new IP's
    if [ -r $USERSOURCE/$USER ]; then
      for ip in `grep -w IP $USERSOURCE/$USER | awk '{print $2}'`; do
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Checking add IP: $ip\" >> $LOG
        fi
        VERIFY="$( echo $ip | grep "@" )"
        if [ "$VERIFY" != "" ]; then
          ## Ok, its really a IP, not part of tagline or so.
          CHECK="$( grep -w IP $USERDEST/$USER | grep -x -F -- "IP $ip" )"
          if [ "$CHECK" = "" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USER - Adding IP $ip"
            fi
            if [ "$LOG" != "" ]; then
              echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - Adding IP $ip\" >> $LOG
            fi
            echo "IP $ip" >> $USERDEST/$USER
          fi
          CHECK=""
          VERIFY=""
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: $USER - Read ip $ip does not contain a @, skipping it\" >> $LOG
          fi
        fi
      done
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read IPs from $USERSOURCE/$USER\" >> $LOG
      fi
    fi
 
    ## Removing redundant ip's
    if [ -r $USERSOURCE/$USER ]; then
      for ip in `grep -w IP $USERDEST/$USER | awk '{print $2}'`; do
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Checking remove IP: $ip\" >> $LOG
        fi
        VERIFY="$( echo $ip | grep "@" )"
        if [ "$VERIFY" != "" ]; then
          ## Ok, its really a IP, not part of tagline or so.
          CHECK="$( grep -w IP $USERSOURCE/$USER | grep -x -F -- "IP $ip" )"
          if [ "$CHECK" = "" ]; then
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USER - Removing IP $ip"
            fi
            if [ "$LOG" != "" ]; then
              echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - Removing IP $ip\" >> $LOG
            fi
            grep -v -x -F -- "IP $ip" $USERDEST/$USER > $TMP/TEMP.$USER.IPDEL
            mv -f $TMP/TEMP.$USER.IPDEL $USERDEST/$USER
          fi
          CHECK=""
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: $USER - Read IP $ip does not contain a $, skipping it\" >> $LOG
          fi
        fi
      done
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: Cant read IPs -remove- from $USERSOURCE/$USER\" >> $LOG
      fi
    fi

    SKIP="NO"
    if [ "$SLOWTIME" != "" ]; then
      sleep $SLOWTIME
    fi
  fi
  SKIP="NO"

  if [ ! -r "$USERSOURCE/$VERIFYUSER" ]; then
    if [ "$DEBUG" = "TRUE" ]; then
      echo "Seems I lost link? Stopping"
    fi
    if [ "$LOG" != "" ]; then
      echo `date "+%a %b %e %T %Y"` USYNC: \"Lost link? Quitting\" >> $LOG
    fi
    rm -f $LOCKFILE
    exit 0
  fi

done

## Delete users from destination, if they are not on source.

EXISTS=""
if [ ! -r $USERSOURCE/$VERIFYUSER ]; then
  if [ "$DEBUG" = "TRUE" ]; then
    echo "Verify user $VERIFYUSER on source does not exist or cant be read. Check link."
  fi
  if [ "$HIGHDEBUG" = "TRUE" ]; then
    echo `date "+%a %b %e %T %Y"` USYNCDB: \"Error: About to check deleted users, but $VERIFYUSER cant be read. Quitting\" >> $LOG
  fi
  rm -f $LOCKFILE
  exit 0
fi

if [ "$DEBUG" = "TRUE" ]; then
  echo "Checking for deleted users."
fi

cd $USERDEST
for USER in `grep -w FLAGS * | awk '{print $1}' | awk -F":" '{print $1}' | egrep -v $EXCLUDE`; do

  if [ "$USERSOURCEORG2" != "" ]; then
    USERSOURCE="$USERSOURCEORG2"
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Switching to $USERSOURCEORG2 to continue checking.\" >> $LOG
    fi
    USERSOURCEORG2=""
  fi

  EXISTS=""
  if [ -e $USERSOURCE/$USER ]; then
    EXISTS="YES"
    if [ "$HIGHDEBUG" = "TRUE" ]; then
      echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user?: $USER = NO.\" >> $LOG
    fi
  else
    EXISTS="NO"
    if [ "$USERSOURCEORG" != "" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user?: $USER = YES. Cant see it on $SOURCENAME. Switching back to $USERSOURCEORG to run more checks.\" >> $LOG
      fi
      USERSOURCEORG2="$USERSOURCE"
      USERSOURCE="$USERSOURCEORG"
    else
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user?: $USER = YES. Cant see it on $SOURCENAME. Running more checks.\" >> $LOG
      fi
    fi
  fi

  if [ "$EXISTS" = "NO" ]; then
    sleep 1
    if [ -r "$USERSOURCE/$VERIFYUSER" ]; then
      if [ "$HIGHDEBUG" = "TRUE" ]; then
        echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: $VERIFYUSER can be read. Link must be up.\" >> $LOG
      fi
      if [ ! -e "$USERSOURCE/$USER" ]; then
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: $USER cant be read on $SOURCENAME - Verifying with passwd.\" >> $LOG
        fi
        PASSWDCHECK=""
        if [ -r "$PASSWDSOURCE/passwd" ]; then
          PASSWDCHECK="$( grep -w $USER $PASSWDSOURCE/passwd )"
          if [ "$PASSWDCHECK" = "" ]; then
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: $USER not found in $PASSWDSOURCE/passwd either.\" >> $LOG
            fi
            if [ "$BACKUPDEST" != "" ]; then
              if [ ! -e "$BACKUPDEST/deleted" ]; then
                if [ "$HIGHDEBUG" = "TRUE" ]; then
                  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Warning: Cant find $BACKUPDEST/delete. Creating it.\" >> $LOG
                fi
                mkdir $BACKUPDEST/deleted
              fi
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - Moved to $BACKUPDEST/deleted/$SUER\" >> $LOG
              fi
              mv -f $USERDEST/$USER $BACKUPDEST/deleted
            else
              if [ "$HIGHDEBUG" = "TRUE" ]; then
                echo `date "+%a %b %e %T %Y"` USYNCDB: \"$USER - No backuppath defined. Deleting userfile.\" >> $LOG
              fi
              rm -f $USERDEST/$USER
            fi
            if [ "$DEBUG" = "TRUE" ]; then
              echo "$USER - Deleted. Not on source anymore"
            fi
            if [ "$LOG" != "" ]; then
              echo `date "+%a %b %e %T %Y"` USYNC: \"$USER - Deleted\" >> $LOG
            fi
          else
            if [ "$HIGHDEBUG" = "TRUE" ]; then
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: $USER FOUND in $PASSWDSOURCE/passwd - No delete!\" >> $LOG
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: Note - Above might mean the passwd and userfiles are not in sync at source site\" >> $LOG
              echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: Note - It can also mean you have a similary named user in the passwd that I found...\" >> $LOG
            fi
          fi
        else
          if [ "$HIGHDEBUG" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` USYNCDB: \"Link lost? Cant read $PASSWDSOURCE/passwd anymore.\" >> $LOG
          fi
        fi
      else
        if [ "$HIGHDEBUG" = "TRUE" ]; then
          echo `date "+%a %b %e %T %Y"` USYNCDB: \"Delete user: $USER found now. No delete!\" >> $LOG
        fi
      fi
    else
      if [ "$DEBUG" = "TRUE" ]; then
        echo "Seems I lost contact. Not deleting $USER"
      fi
      if [ "$LOG" != "" ]; then
        echo `date "+%a %b %e %T %Y"` USYNC: \"Lost link I think. Not deleting $USER\" >> $LOG
      fi
    fi      
  fi
done

if [ "$HIGHDEBUG" = "TRUE" ]; then
  echo `date "+%a %b %e %T %Y"` USYNCDB: \"Done! Clearing $LOCKFILE and quitting.\" >> $LOG
fi
rm -f $LOCKFILE
exit 0
