#!/bin/bash
VER=1.1

COMMAND="transfer"

SOURCENAME="HUB"
USERSOURCEROOT=/glftpd
USERSOURCE=/ftp-data/users.ext

DESTNAME="SLV"
USERDESTROOT=/glftpd
USERDEST=/ftp-data/users

LOG=/ftp-data/logs/transfers.log
TMP=/tmp
BIN=/bin
MINAMOUNT="10"
MAXAMOUNT="10000"


###########################
# GO                      #
###########################

## Shell or glftpd?
if [ "$FLAGS" = "" ]; then
  MODE="SHELL"
  USERSOURCE="$USERSOURCEROOT$USERSOURCE"
  USERDEST="$USERDESTROOT$USERDEST"
  LOG="$USERDESTROOT$LOG"
  TMP="$USERDESTROOT$TMP"
  BIN="$USERDESTROOT$BIN"
  TAKE="$1"
  USER="$2"
  CREDS="$3"
else
  MODE="GL"
  TAKE="$1"
  CREDS="$2"
fi

## Some aliases
if [ "$TAKE" = "put" -o "$TAKE" = "throw" ]; then
  TAKE="give"
fi
if [ "$TAKE" = "get" -o "$TAKE" = "fetch" ]; then
  TAKE="take"
fi
if [ "$TAKE" = "TAKE" ]; then
  TAKE="take"
fi
if [ "$TAKE" = "GIVE" ]; then
  TAKE="give"
fi

echo "Please wait. Verifying network. - Mode = $MODE"
  
if [ "$MODE" = "SHELL" ]; then
  MOUNT1="$( $BIN/df -P | grep -w -F -- $USERSOURCE )"
  if [ "$MOUNT1" = "" ]; then
    echo "Seems the source files are not mounted. Talk to a siteop."
    exit 0
  fi
fi

if [ -e "$USERDEST/$USER" ]; then
  ok=yes
else
  echo "User $USER does not exist on $DESTNAME or permissions are wrong."
  exit 0
fi

if [ -e "$USERSOURCE/$USER" ]; then
  ok=yes
else
  echo "User $USER does not exist on $SOURCENAME or link is down."
  exit 0 
fi

## Verify network link
if [ -w "$USERSOURCE/$USER" ]; then
  LINK="UP"
else
  echo "$USERSOURCE/$USER"
  echo "Transfer failed. Link to source failed or the userfile is not writable."
  exit 0
fi
if [ -w "$USERDEST/$USER" ]; then
  LINK="UP"
else
  echo "Transfer failed. Cant write to current userfile."
  exit 0
fi

if [ "$MODE" = "GL" -a "$1" = "status" ]; then
  if [ "$2" != "" ]; then
    USER="$2"
  fi
  SOURCECREDSKB="$( grep -w CREDITS $USERSOURCE/$USER | awk '{print $2}' )"
  if [ "$SOURCECREDSKB" = "" ]; then
    echo "Can not read $SOURCENAME credits for $USER"
    exit 0
  else
    SOURCECREDSMB="$( expr $SOURCECREDSKB \/ 1024 )"  
  fi
  DESTCREDSKB="$( grep -w CREDITS $USERDEST/$USER | awk '{print $2}' )"
  if [ "$DESTCREDSKB" = "" ]; then
    echo "Can not read $DESTNAME credits for $USER"
    exit 0
  else
    DESTCREDSMB="$( expr $DESTCREDSKB \/ 1024 )"
  fi
  echo "-----------------------------------------"
  echo "Current credits:                         "
  echo "$SOURCENAME : $SOURCECREDSMB Mb"
  echo "$DESTNAME : $DESTCREDSMB Mb"
  echo "-----------------------------------------"
  exit 0
fi

if [ "$CREDS" = "" ]; then
  echo "-------------------------------------------------"
  echo "Welcome to the Credits Transfer System $VER      "
  echo "Current link is between $SOURCENAME/$DESTNAME"
  echo "-------------------------------------------------"
  echo "Usage :                                          "
  if [ "$MODE" = "GL" ]; then
    echo "site $COMMAND status                             "
    echo "site $COMMAND take/give Mb_Amount                "
   else
    echo "transfercredits.sh take/give username Mb_Amount  "
  fi
  echo "take = $SOURCENAME -> $DESTNAME"
  echo "give = $DESTNAME -> $SOURCENAME"
  echo "                                                 "
  echo "Min amount to move is $MINAMOUNT Mb.             "
  echo "Max amount to move is $MAXAMOUNT Mb.             "
  echo "Specifying 'all' instead of amount will move all "
  echo "your credits between sites, but not over max     "
  echo "amount though.                                   "
  echo ""
  echo "Example:"
  if [ "$MODE" = "GL" ]; then
    echo "site $COMMAND take 500"
    echo "would move 500 meg creds from $SOURCENAME to $DESTNAME"
  else
    echo "site $COMMAND take turranius 500"
    echo "would move 500 meg of turranius's creds from $SOURCENAME to $DESTNAME"
  fi
  echo "---------------------------[ Turranius 2002 ]----"
  exit 0
fi

## Make sure TAGLINE does not contain any of the words in the below egrep.
VERIFY=""
VERIFY="$( grep -w TAGLINE $USERSOURCE/$USER | egrep -w 'ALLUP|ALLDN|WKUP|WKDN|DAYUP|DAYDN|MONTHUP|MONTHDN|NUKE|TIME|CREDITS' )"
if [ "$VERIFY" != "" ]; then
  echo "Your tagline on $SOURCENAME contains a bad word. Please change it and wait for it to be replicated."
  echo "Bad words are: ALLUP, ALLDN, WKUP, WKDN, DAYUP, DAYDN, MONTHUP, MONTHDN, NUKE, TIME & CREDITS"
  exit 0
fi
VERIFY=""
VERIFY="$( grep -w TAGLINE $USERDEST/$USER | egrep -w 'ALLUP|ALLDN|WKUP|WKDN|DAYUP|DAYDN|MONTHUP|MONTHDN|NUKE|TIME|CREDITS' )"
if [ "$VERIFY" != "" ]; then
  echo "Your tagline on $DESTNAME contains a bad word. Please change it and wait for it to be replicated."
  echo "Bad words are: ALLUP, ALLDN, WKUP, WKDN, DAYUP, DAYDN, MONTHUP, MONTHDN, NUKE, TIME & CREDITS"
  exit 0
fi

if [ "$CREDS" = "all" -o "$CREDS" = "ALL" ]; then
  CREDS="666"
  ALL="TRUE" 
fi

## Make sure bet is not over 6 chars long
MAX="$( echo "$CREDS" | wc -c | tr -d ' ' )"
if [ "$MAX" -gt "6" ]; then
  echo "Nice try to overflow, but max in $MAXAMOUNT Mb, sucka."
  exit 0
fi

## Make a test calculation on it to make sure there are no crap in it.
TESTING="$( expr $CREDS \+ 2 )"
if [ "$TESTING" = "" ]; then
  echo "Only use numbers in credits. No dots or other chars."
  exit 0
fi

## Check for -
VERIFY="$( echo $CREDS | grep -F -i -- "-" )"
if [ "$VERIFY" != "" ]; then
  echo "No can do buddy! Done use -"
  exit 0
fi

## User tried to transfer minus credits.
if [ "$CREDS" -lt "0" ]; then
  echo "Nice try, jackass"
  exit 0
fi

## User tried to transfer below minumum limit.
if [ "$CREDS" -lt "$MINAMOUNT" ]; then
  echo "Minimum amount to transfer is $MINAMOUNT Mb"
  exit 0
fi 

## User tried to transfer above max limit.
if [ "$CREDS" -gt "$MAXAMOUNT" ]; then
  echo "Maximum amount to transfer is $MAXAMOUNT Mb"
  exit 0
fi 

if [ "$LINK" = "UP" ]; then
  
  if [ "$TAKE" != "give" ]; then
    if [ "$TAKE" != "take" ]; then
      echo "Use either give or take."
      if [ "$MODE" = "GL" ]; then
        echo "site $COMMAND, for help"
      fi
      exit 0
    fi
  fi
 
  ## Put bet in variable $CREDSMB and make a KiloByte calculation on it.
  CREDSMB="$CREDS"
  CREDSKB="$( expr $CREDSMB \* 1024 )"

  ## Try and read credits from source site.
  SOURCECREDSKB="$( grep -w CREDITS $USERSOURCE/$USER | awk '{print $2}' )"
  if [ "$SOURCECREDSKB" = "" ]; then
    echo "Can not read source sites credits."
    exit 0
  fi
  SOURCECREDSMB="$( expr $SOURCECREDSKB \/ 1024 )"
  
  ## Try and read credits from destination site.
  DESTCREDSKB="$( grep -w CREDITS $USERDEST/$USER | awk '{print $2}' )"
  if [ "$DESTCREDSKB" = "" ]; then
    echo "Can not read destination sites credits."
    exit 0
  fi
  DESTCREDSMB="$( expr $DESTCREDSKB \/ 1024 )"
  
  ## --> Take <-- credits from source
  if [ "$TAKE" = "take" ]; then
    if [ "$ALL" = "TRUE" -a "$CREDS" = "666" ]; then
      CREDSMB="$SOURCECREDSMB"
      CREDSKB="$( expr $CREDSMB \* 1024 )"
      if [ "$CREDSMB" -lt "$MINAMOUNT" ]; then
        echo "You must have atleast $MINAMOUNT Mb credits to move."
        exit 0
      else
        if [ "$CREDSMB" -gt "$MAXAMOUNT" ]; then
          echo "Max amount is $MAXAMOUNT Mb. Setting it to that amount."
          CREDSMB="$MAXAMOUNT"
          CREDSKB="$( expr $CREDSMB \* 1024 )"
        else
          echo "Moving ALL credits ($SOURCECREDSMB Mb) from $SOURCENAME to $DESTNAME."
        fi
      fi
    fi  

    if [ "$CREDSKB" -gt "$SOURCECREDSKB" ]; then
      echo "You do not have $CREDSMB Mb on $SOURCENAME - Only $SOURCECREDSMB Mb"
      exit 0
    else
      ## Check that no .lock file exists on source.
      if [ -e $USERSOURCE/$USER.lock ]; then
        echo "Your userfile is locked on $SOURCENAME - Log out for secure transfer or try again."
        exit 0
      fi

      ## Check that no .lock file exists on destination.
      if [ -e $USERDEST/$USER.lock ]; then
        echo "Your userfile is locked on $DESTNAME - Log out any other sessions or try again."
        exit 0
      fi

      echo "Transfering credits. Do NOT log out."
      NEWCREDSDESTKB="$( expr $DESTCREDSKB \+ $CREDSKB )"
      NEWCREDSDESTMB="$( expr $NEWCREDSDESTKB \/ 1024 )"
      NEWCREDSSOURCEKB="$( expr $SOURCECREDSKB \- $CREDSKB )"
      NEWCREDSSOURCEMB="$( expr $NEWCREDSSOURCEKB \/ 1024 )"
      if [ "$ALL" != "TRUE" ]; then
        echo "Transfering $CREDSKB Kb ($CREDSMB Mb) from $SOURCENAME to $DESTNAME"
      fi
      if [ -w "$USERSOURCE/$USER" ]; then

        ## Log it.
        if [ "$LOG" != "" ]; then
          if [ "$ALL" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` TRANS: \"$USER transfers ALL his creds, $CREDSMB Mb, from $SOURCENAME to $DESTNAME\" >> $LOG
          else
            echo `date "+%a %b %e %T %Y"` TRANS: \"$USER transfers $CREDSMB Mb from $SOURCENAME to $DESTNAME\" >> $LOG
          fi
        fi

        ## Give credits to source.
        if [ -w $USERSOURCE/$USER ]; then
          if [ -w $USERDEST/$USER ]; then
            sed -e "s/^CREDITS.*/CREDITS $NEWCREDSSOURCEKB/" $USERSOURCE/$USER > $TMP/TEMP.$USER.CREDITS
            cp -f $TMP/TEMP.$USER.CREDITS $USERSOURCE/$USER

            ## Take credits from destination.
            sed -e "s/^CREDITS.*/CREDITS $NEWCREDSDESTKB/" $USERDEST/$USER > $TMP/TEMP.$USER.CREDITS
            cp -f $TMP/TEMP.$USER.CREDITS $USERDEST/$USER
          else
            echo "Cant write destination file $USERDEST/$USER - Quitting" 
            exit 0
          fi
        else
          echo "Cant write source file $USERSOURCE/$USER - Quitting"
          exit 0
        fi
      else
        echo "Hrmp, seems I cant write to userfile. Either link went down or perms are wrong."
        exit 0
      fi
      echo "All done. Please verify both sites BEFORE you use this command again."
      echo "( you can verify with: site $COMMAND status )"
      exit 0
    fi
  fi

  ## Give credits to source.
  if [ "$TAKE" = "give" ]; then

    if [ "$ALL" = "TRUE" -a "$CREDS" = "666" ]; then
      CREDSMB="$DESTCREDSMB"
      CREDSKB="$( expr $CREDSMB \* 1024 )"
      if [ "$CREDSMB" -lt "$MINAMOUNT" ]; then
        echo "You must have atleast $MINAMOUNT Mb credits to move."
        exit 0
      else
        if [ "$CREDSMB" -gt "$MAXAMOUNT" ]; then
          echo "Max amount is $MAXAMOUNT Mb. Setting it to that amount."
          CREDSMB="$MAXAMOUNT"
          CREDSKB="$( expr $CREDSMB \* 1024 )"
        else
          echo "Moving ALL credits ($DESTCREDSMB Mb) from $DESTNAME to $SOURCENAME."
        fi
      fi
    fi  

    if [ "$CREDSKB" -gt "$DESTCREDSKB" ]; then
      echo "You do not have $CREDSMB Mb on $DESTNAME - Only $DESTCREDSMB Mb"
      exit 0
    else
      if [ -e $USERSOURCE/$USER.lock ]; then
        echo "Your userfile is locked on $SOURCENAME - Log out for secure transfer or try again."
        exit 0
      fi

      ## Check that no .lock file exists on destination.
      if [ -e $USERDEST/$USER.lock ]; then
        echo "Your userfile is locked on $DESTNAME - Log out any other sessions or try again."
        exit 0
      fi

      echo "Transfering credits. Do NOT log out."
      NEWCREDSDESTKB="$( expr $DESTCREDSKB \- $CREDSKB )"
      NEWCREDSDESTMB="$( expr $NEWCREDSDESTKB \/ 1024 )"
      NEWCREDSSOURCEKB="$( expr $SOURCECREDSKB \+ $CREDSKB )"
      NEWCREDSSOURCEMB="$( expr $NEWCREDSSOURCEKB \/ 1024 )"
      if [ "$ALL" != "TRUE" ]; then
        echo "Transfering $CREDSKB Kb ($CREDSMB Mb) from $DESTNAME to $SOURCENAME"
      fi
      if [ -w "$USERSOURCE/$USER" ]; then

        ## Log it.
        if [ "$LOG" != "" ]; then
          if [ "$ALL" = "TRUE" ]; then
            echo `date "+%a %b %e %T %Y"` TRANS: \"$USER transfers ALL his creds, $CREDSMB Mb, from $DESTNAME to $SOURCENAME\" >> $LOG
          else
            echo `date "+%a %b %e %T %Y"` TRANS: \"$USER transfers $CREDSMB Mb from $DESTNAME to $SOURCENAME\" >> $LOG
          fi
        fi

        if [ -w $USERSOURCE/$USER ]; then
          if [ -w $USERDEST/$USER ]; then
            ## Take credits from source.
            sed -e "s/^CREDITS.*/CREDITS $NEWCREDSSOURCEKB/" $USERSOURCE/$USER > $TMP/TEMP.$USER.CREDITS
            cp -f $TMP/TEMP.$USER.CREDITS $USERSOURCE/$USER

            ## Give credits to destination.
            sed -e "s/^CREDITS.*/CREDITS $NEWCREDSDESTKB/" $USERDEST/$USER > $TMP/TEMP.$USER.CREDITS
            cp -f $TMP/TEMP.$USER.CREDITS $USERDEST/$USER
          else
            echo "Cant write to destination file $USERDEST/$USER - Quitting"
            exit 0
          fi
        else
          echo "Cant write to source file $USERSOURCE/$USER - Quitting"
          exit 0
        fi
      else
        echo "Hm, seems I cant write to userfiles anymore. Either link is down or perms are wrong."
        exit 0
      fi
      echo "All done. Please verify both sites BEFORE you use this command again."
      echo "( you can verify with: site $COMMAND status )"
      exit 0
    fi
  fi

fi